
% Prepare figures
figure(1), clf, hold on, axis equal
axis([0 153 -3 60]);
daspect([1 1 1]);
title('Recursive position estimate during initialization','interpreter','latex','fontsize',12)
ylabel('position y [m]','interpreter','latex','fontsize',12)
xlabel('position x [m]','interpreter','latex','fontsize',12)
set(gca,'YTick',[0 25 50]);
figure(2), clf, xlim([-40,80]), axis equal, grid on
figure(3), clf
figure(4), clf, hold on
edges = (-180:1:180)*pi/180;
plot_hyp = true;

% Initial state estimates
x0 = zeros(8,1);
P0 = zeros(4);
x0(1:2)=[0;15];

gamma2=1;%0.5;
alpha=1;

% Create dead reckoning data and range measurmements
x0_ref = [[[0;15;0]  [6;30;0]  [12;32;0] [18;33;0] [23;38;0] [38;37;0] [42;40;0] [50;44;0] [55;46;0] [59;48;0] [67;47;0] [70;43;0] [75;40;0] [83;36;0] [89;34;0] [91;33;0]  [100;32;0]  [105;28;0]  [108;24;0]  [116;22;0]  [134;23;0] [141;26;0] [146;30;0] [150;33;0]];
          [0          0         0        -10       -10     45      45      45      5       5       5       -25     -25     -25      -5      -5       -5       -15      -15       -15      5         5       5       45]*pi/180];
x1_ref = [[[16;13;0] [29;12;0] [35;13;0] [41;16;0] [47;19;0] [62;21;0] [64;20;0] [79;14;0] [80;13;0] [87;8;0]  [95;9;0]  [96;10;0] [105;9;0]  [111;15;0] [114;16;0] [123;23;0] [127;35;0] [129;37;0] [131;40;0] [135;45;0] [137;46;0] [141;50;0] [146;52;0] [150;53;0]];
          [0        0       0       5       5       45      45      45      -15     -15     -15     0       0       0        50      50       50       45       45        45       55        55      55      45]*pi/180];
ranging_done = [0        0       0       1       0       1       0       0       1       0       0       1       0       0        1       0        0        1        0         0        1         0       0       1     ];
[dx r] = create_sim_data(x0_ref,x1_ref);

% Plot true trajectories
figure(1)
plot(x1_ref(1,:),x1_ref(2,:),'k:','LineWidth',1.5);

% Base hypothesis
r_hyp = [-0.5 0 0.5];%[-0.5 0.2 0 0.2 0.5];%
h_hyp = [-0.5 0 0.5];
dori = 360/32;%36;%
chi_hyp = (0:dori:360-dori)'*pi/180;
theta_hyp = (0:dori:360-dori)'*pi/180;
N_hyp = numel(r_hyp)*numel(h_hyp)*numel(chi_hyp)*numel(theta_hyp);
z=zeros(6,N_hyp);
w=zeros(1,N_hyp);
fprintf('Number of particles: %i\n',N_hyp);

% System states
x = x0;
P = zeros(8);
Pr0 = zeros(8,4);
x_dummy = zeros(4,1);
x_old = x;

N = size(dx,2);
init=0;
nr_rng=0;
x_save=zeros(numel(x),N);
for i=1:N    
    % Update dead reckoning
    for j=1:2
        [x P] = swdrN_tud(x,P,dx((j-1)*4+1:j*4,i),diag([(0.01*dx((j-1)*4+1:j*4-2,i)).^2; 0.1;0.01^2]),(j-1)*4+1);
    end
    % Update relative covariance
    [x_dummy Pr0(5:8,:)] = swdrN_tud(x(5:8),Pr0(5:8,:),dx((j-1)*4+1:j*4,i),diag([(0.01*dx((j-1)*4+1:j*4-2,i)).^2; 0.1;0.01^2]),1);
    
    if ranging_done(i+1)
        [x00 Pr00] = to_sample_frame(x(5:8),x0(5:8),Pr0(5:8,:));
        if nr_rng==0
            [z w] = drrng_init_hyp(x(1:3),x00,r(i+1),r_hyp,h_hyp,chi_hyp,theta_hyp);
        else
            w = drrng_reweight(z,w,x(1:3),r(i+1),x00,gamma2);
        end
        [x0(5:8) P0] = hyp_mean_cov(z,w);
        [z w] = drrng_resample(z,w,x0(5:8),P0,alpha);
        [x(5:8) P(5:8,5:8) Pr0(5:8,:)] = to_global_frame(x00,x0(5:8),Pr00,P0);

        
        % Compute the effective number of particles
        N_eff = 1/sum(w.^2);
        fprintf('Effective nr of particles: %i\n',round(N_eff));
        % Display results
        fprintf('Current estimate: [%1.1f %1.1f %1.1f %1.1f]\n',x0(5),x0(6),x0(7),x0(8));
        fprintf('Current variance: [%1.1f %1.1f %1.1f %1.1f]\n',sqrt(P0(1,1)),sqrt(P0(2,2)),sqrt(P0(3,3)),sqrt(P0(4,4))*180/pi );
        
        % Plot things
        if plot_hyp
            figure(2), clf
            ind = w > 0.99*1/N_hyp;
            scatter(z(1,ind),z(2,ind));
            axis([-60 89 -20 100]), grid on, hold on, box on
            [C ml] = max(w);
            plot(z(1,ml),z(2,ml),'gd','LineWidth',3,'MarkerSize',15)
            plot(x1_ref(1,1),x1_ref(2,1),'r+','LineWidth',3,'MarkerSize',15)
            plot(x0(5),x0(6),'kx','LineWidth',3,'MarkerSize',15)
            err_el=error_ellipse(P0(1:2,1:2),x0(5:6),'style','k','conf', 0.68);
            set(err_el,'LineWidth',3);
            title(['Particle cloude of $$\hat{\mathbf{x}}_{0|' sprintf('%i',nr_rng) '}$$'],'interpreter','latex','fontsize',16)
            ylabel('Position y [m]','interpreter','latex','fontsize',12)
            xlabel('Position x [m]','interpreter','latex','fontsize',12)
            set(gcf, 'PaperPositionMode','auto')
            filename = sprintf('particle_cloud_%i',nr_rng);
%             saveas(gcf,['./Figures/' filename '.eps'],'epsc')
%             eps2pdf(['./Figures/' filename '.eps'])
%             copyfile(['./Figures/' filename '.pdf'],'./../Figures/')
            figure(3), clf
            bins = hist(mod(z(4,ind)+pi,2*pi)-pi,edges(1:end-1)+pi/numel(edges));
            bins = bins/max(bins);
            bar(edges(1:end-1)+pi/numel(edges),bins)
            ylabel('Rel. freq.','interpreter','latex','fontsize',12)
            xlabel('Initial heading [rad]','interpreter','latex','fontsize',12)
            xlim([edges(1) edges(end)]);
            dar=get(gca,'DataAspectRatio');
            set(gca,'DataAspectRatio',[dar(1)*0.15 dar(2) dar(3)]);
            set(gca,'XTick',[-pi 0 pi],'XTickLabel',{'-3.14';'0';'3.14'},'YTick',[0 1]);
            set(gcf, 'PaperPositionMode','auto')
            filename2 = sprintf('ori_hist_%i',nr_rng);
%             saveas(gcf,['./Figures/' filename2 '.eps'],'epsc')
%             eps2pdf(['./Figures/' filename2 '.eps'])
%             copyfile(['./Figures/' filename2 '.pdf'],'./../Figures/')
        end
        
        figure(1)
        plot(x(1),x(2),'bo','LineWidth',1,'MarkerSize',10);
        plot([x(1)+2.5*cos(x(4)) x(1)+6*cos(x(4))],[x(2)+2.5*sin(x(4)) x(2)+6*sin(x(4))],'b','LineWidth',1.2)
        plot(x(5),x(6),'ro','LineWidth',1,'MarkerSize',10);
        plot([x(5)+2.5*cos(x(8)) x(5)+6*cos(x(8))],[x(6)+2.5*sin(x(8)) x(6)+6*sin(x(8))],'r','LineWidth',1.2)
        
        nr_rng=nr_rng+1;
    end
    
    x_save(:,i)=x;
    
    figure(1)
    plot([x_old(1) x(1)],[x_old(2) x(2)],'b--');
    plot([x_old(5) x(5)],[x_old(6) x(6)],'r--');
    x_old=x;
    drawnow
%    pause(1)
end
figure(4)
error=x_save(5:8,:)-x1_ref(:,2:end);
error = sqrt(sum(error(1:3,4:end).^2,1));
plot(1:numel(error)-1,error(2:end),'LineWidth',1.5);
title('Position error for different number of particles','interpreter','latex','fontsize',10)
ylabel('Position error [m]','interpreter','latex','fontsize',10)
xlabel('Sample index $$k$$','interpreter','latex','fontsize',10)
box on, grid on
dar=get(gca,'DataAspectRatio');
set(gca,'DataAspectRatio',[dar(1)*0.4 dar(2) dar(3)]);

% set(gcf, 'PaperPositionMode','auto')
% filename = 'pos_err_nr_hyp';
% saveas(gcf,['./Figures/' filename '.eps'],'epsc')
% eps2pdf(['./Figures/' filename '.eps'])
% copyfile(['./Figures/' filename '.pdf'],'./../Figures/')

% figure(1)
% set(gcf, 'PaperPositionMode','auto')
% filename = 'recursive_position_estimate';
% saveas(gcf,['./Figures/' filename '.eps'],'epsc')
% eps2pdf(['./Figures/' filename '.eps'])
% copyfile(['./Figures/' filename '.pdf'],'./../Figures/')